@@ -35,15 +35,16 @@
 #include <gtk/gtk.h>
 #include <dbus/dbus-glib.h>
 #include <dbus/dbus-glib-lowlevel.h>
-#include <glade/glade.h>
 
 #include "gpm-stock-icons.h"
 #include "gpm-common.h"
-#include "egg-debug.h"
-
 #include "gpm-manager.h"
+#include "gpm-session.h"
+
 #include "org.freedesktop.PowerManagement.h"
 
+#include "egg-debug.h"
+
 /**
  * gpm_object_register:
  * @connection: What we want to register to
@@ -114,6 +115,38 @@ timed_exit_cb (GMainLoop *loop)
 	return FALSE;
 }
 
+/**
+ * gpm_main_stop_cb:
+ **/
+static void
+gpm_main_stop_cb (GpmSession *session, GMainLoop *loop)
+{
+	g_main_loop_quit (loop);
+}
+
+/**
+ * gpm_main_query_end_session_cb:
+ **/
+static void
+gpm_main_query_end_session_cb (GpmSession *session, guint flags, GMainLoop *loop)
+{
+	/* just send response */
+	gpm_session_end_session_response (session, TRUE, NULL);
+}
+
+/**
+ * gpm_main_end_session_cb:
+ **/
+static void
+gpm_main_end_session_cb (GpmSession *session, guint flags, GMainLoop *loop)
+{
+	/* send response */
+	gpm_session_end_session_response (session, TRUE, NULL);
+
+	/* exit loop, will unref manager */
+	g_main_loop_quit (loop);
+}
+
 /**
  * main:
  **/
@@ -127,6 +160,7 @@ main (int argc, char *argv[])
 	gboolean version = FALSE;
 	gboolean timed_exit = FALSE;
 	gboolean immediate_exit = FALSE;
+	GpmSession *session = NULL;
 	GpmManager *manager = NULL;
 	GError *error = NULL;
 	GOptionContext *context;
@@ -159,7 +193,6 @@ main (int argc, char *argv[])
 	g_option_context_set_translation_domain(context, GETTEXT_PACKAGE);
 	g_option_context_set_summary (context, _("GNOME Power Manager"));
 	g_option_context_parse (context, &argc, &argv, NULL);
-	g_option_context_free (context);
 
 	if (version) {
 		g_print ("Version %s\n", VERSION);
@@ -200,6 +233,15 @@ main (int argc, char *argv[])
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
                                            GPM_DATA G_DIR_SEPARATOR_S "icons");
 
+	loop = g_main_loop_new (NULL, FALSE);
+
+	/* optionally register with the session */
+	session = gpm_session_new ();
+	g_signal_connect (session, "stop", G_CALLBACK (gpm_main_stop_cb), loop);
+	g_signal_connect (session, "query-end-session", G_CALLBACK (gpm_main_query_end_session_cb), loop);
+	g_signal_connect (session, "end-session", G_CALLBACK (gpm_main_end_session_cb), loop);
+	gpm_session_register_client (session, "gnome-power-manager", getenv ("DESKTOP_AUTOSTART_ID"));
+
 	/* create a new gui object */
 	manager = gpm_manager_new ();
 
@@ -208,8 +250,6 @@ main (int argc, char *argv[])
 		return 0;
 	}
 
-	loop = g_main_loop_new (NULL, FALSE);
-
 	/* Only timeout and close the mainloop if we have specified it
 	 * on the command line */
 	if (timed_exit) {
@@ -222,6 +262,7 @@ main (int argc, char *argv[])
 
 	g_main_loop_unref (loop);
 
+	g_object_unref (session);
 	g_object_unref (manager);
 unref_program:
 	g_option_context_free (context);
