@@ -40,18 +40,16 @@
 #include <dbus/dbus-glib-lowlevel.h>
 #include <gconf/gconf-client.h>
 #include <canberra-gtk.h>
-#include <dkp-device.h>
+#include <devkit-power-gobject/devicekit-power.h>
 
 #include "egg-debug.h"
 #include "egg-console-kit.h"
 
-#include "gpm-ac-adapter.h"
 #include "gpm-button.h"
 #include "gpm-control.h"
 #include "gpm-common.h"
 #include "gpm-dpms.h"
 #include "gpm-idle.h"
-#include "gpm-inhibit.h"
 #include "gpm-manager.h"
 #include "gpm-notify.h"
 #include "gpm-prefs.h"
@@ -67,7 +65,6 @@
 #include "gpm-devicekit.h"
 #include "gpm-feedback-widget.h"
 
-#include "org.freedesktop.PowerManagement.Inhibit.h"
 #include "org.freedesktop.PowerManagement.Backlight.h"
 
 static void     gpm_manager_finalize	(GObject	 *object);
@@ -77,19 +74,16 @@ static void     gpm_manager_finalize	(GObject	 *object);
 
 struct GpmManagerPrivate
 {
-	GpmAcAdapter		*ac_adapter;
 	GpmButton		*button;
 	GConfClient		*conf;
 	GpmDpms			*dpms;
 	GpmIdle			*idle;
 	GpmPrefsServer		*prefs_server;
-	GpmInhibit		*inhibit;
 	GpmNotify		*notify;
 	GpmControl		*control;
 	GpmScreensaver 		*screensaver;
 	GpmTrayIcon		*tray_icon;
 	GpmEngine		*engine;
-	gboolean		 low_power;
 	GpmBrightnessKbd	*brightness_kbd;
 	GpmFeedback		*feedback_kbd;
 	GpmBacklight		*backlight;
@@ -97,13 +91,8 @@ struct GpmManagerPrivate
 	guint32         	 screensaver_ac_throttle_id;
 	guint32         	 screensaver_dpms_throttle_id;
 	guint32         	 screensaver_lid_throttle_id;
-};
-
-enum {
-	ON_BATTERY_CHANGED,
-	LOW_BATTERY_CHANGED,
-	POWER_SAVE_STATUS_CHANGED,
-	LAST_SIGNAL
+	DkpClient		*client;
+	gboolean		 on_battery;
 };
 
 typedef enum {
@@ -120,8 +109,6 @@ typedef enum {
 	GPM_MANAGER_SOUND_LAST
 } GpmManagerSound;
 
-static guint	     signals [LAST_SIGNAL] = { 0 };
-
 G_DEFINE_TYPE (GpmManager, gpm_manager, G_TYPE_OBJECT)
 
 /**
@@ -168,7 +155,7 @@ gpm_manager_play (GpmManager *manager, GpmManagerSound action, gboolean force)
 	const gchar *desc = NULL;
 	gboolean ret;
 
-	ret = gconf_client_get_bool (manager->priv->conf, GPM_CONF_UI_ENABLE_BEEPING, NULL);
+	ret = gconf_client_get_bool (manager->priv->conf, GPM_CONF_UI_ENABLE_SOUND, NULL);
 	if (!ret && !force) {
 		egg_debug ("ignoring sound due to policy");
 		return FALSE;
@@ -235,53 +222,18 @@ gpm_manager_play (GpmManager *manager, GpmManagerSound action, gboolean force)
  * @action: The action we want to do, e.g. "suspend"
  *
  * Checks to see if the specific action has been inhibited by a program.
- * If so, displays a warning libnotify dialogue for the user explaining
- * the situation.
  *
  * Return value: TRUE if we can perform the action.
  **/
 static gboolean
 gpm_manager_is_inhibit_valid (GpmManager *manager, gboolean user_action, const char *action)
 {
-	gboolean has_inhibit;
-	gchar *title = NULL;
-
-	/* We have to decide on whether this is a idle action or a user keypress */
-	gpm_inhibit_has_inhibit (manager->priv->inhibit, &has_inhibit, NULL);
-
-	if (has_inhibit) {
-		GString *message = g_string_new ("");
-		const char *msg;
-
-		/*Compose message for each possible action*/
-		if (strcmp (action, "suspend") == 0)
-				title = g_strdup (_("Request to suspend")); 
-		else if (strcmp (action, "hibernate") == 0)
-				title = g_strdup (_("Request to hibernate")); 
-		else if (strcmp (action, "policy action") == 0)
-				title = g_strdup (_("Request to do policy action")); 
-		else if (strcmp (action, "timeout action") == 0)
-				title = g_strdup (_("Request to do timeout action"));
-		
-		gpm_inhibit_get_message (manager->priv->inhibit, message, action);
-		gpm_notify_display (manager->priv->notify,
-				      title,
-				      message->str,
-				      GPM_NOTIFY_TIMEOUT_LONG,
-				      GTK_STOCK_DIALOG_WARNING,
-				      GPM_NOTIFY_URGENCY_NORMAL);
-		/* I want this translated */
-		msg = _("Perform action anyway");
-		g_string_free (message, TRUE);
-		g_free (title);
-	}
-	return !has_inhibit;
+	return TRUE;
 }
 
 /**
  * gpm_manager_sync_policy_sleep:
  * @manager: This class instance
- * @on_ac: If we are on AC power
  *
  * Changes the policy if required, setting brightness, display and computer
  * timeouts.
@@ -293,19 +245,13 @@ gpm_manager_sync_policy_sleep (GpmManager *manager)
 {
 	guint sleep_display;
 	guint sleep_computer;
-	gboolean on_ac;
-	gboolean power_save;
 
-	on_ac = gpm_ac_adapter_is_present (manager->priv->ac_adapter);
-
-	if (on_ac) {
+	if (!manager->priv->on_battery) {
 		sleep_computer = gconf_client_get_int (manager->priv->conf, GPM_CONF_TIMEOUT_SLEEP_COMPUTER_AC, NULL);
 		sleep_display = gconf_client_get_int (manager->priv->conf, GPM_CONF_TIMEOUT_SLEEP_DISPLAY_AC, NULL);
-		power_save = gconf_client_get_bool (manager->priv->conf, GPM_CONF_LOWPOWER_AC, NULL);
 	} else {
 		sleep_computer = gconf_client_get_int (manager->priv->conf, GPM_CONF_TIMEOUT_SLEEP_COMPUTER_BATT, NULL);
 		sleep_display = gconf_client_get_int (manager->priv->conf, GPM_CONF_TIMEOUT_SLEEP_DISPLAY_BATT, NULL);
-		power_save = gconf_client_get_bool (manager->priv->conf, GPM_CONF_LOWPOWER_BATT, NULL);
 	}
 
 	/* set the new sleep (inactivity) value */
@@ -584,57 +530,6 @@ gpm_manager_get_preferences_options (GpmManager *manager, gint *capability, GErr
 	return gpm_prefs_server_get_capability (manager->priv->prefs_server, capability);
 }
 
-/**
- * gpm_manager_get_power_save_status:
- *
- * Returns true if low power mode has been set.
- * This may be set on AC or battery power, both, or neither depending on
- * the users policy setting.
- * This setting may also change with the battery level changing.
- * Programs should respect this value for the session.
- **/
-gboolean
-gpm_manager_get_power_save_status (GpmManager *manager, gboolean *low_power, GError **error)
-{
-	g_return_val_if_fail (manager != NULL, FALSE);
-	g_return_val_if_fail (GPM_IS_MANAGER (manager), FALSE);
-	*low_power = manager->priv->low_power;
-	return TRUE;
-}
-
-/**
- * gpm_manager_get_on_battery:
- *
- * Returns the system AC state, i.e. if we are not running on battery
- * power.
- * Note: This method may still return false on AC using a desktop system
- * if the computer is using backup power from a monitored UPS.
- **/
-gboolean
-gpm_manager_get_on_battery (GpmManager *manager, gboolean *on_battery, GError **error)
-{
-	gboolean on_ac;
-	g_return_val_if_fail (manager != NULL, FALSE);
-	g_return_val_if_fail (GPM_IS_MANAGER (manager), FALSE);
-	on_ac = gpm_ac_adapter_is_present (manager->priv->ac_adapter);
-	*on_battery = !on_ac;
-	return TRUE;
-}
-
-/**
- * gpm_manager_get_low_battery:
- **/
-gboolean
-gpm_manager_get_low_battery (GpmManager *manager, gboolean *low_battery, GError **error)
-{
-	g_return_val_if_fail (manager != NULL, FALSE);
-	g_return_val_if_fail (GPM_IS_MANAGER (manager), FALSE);
-
-	/* TODO */
-	*low_battery = FALSE;
-	return TRUE;
-}
-
 /**
  * idle_do_sleep:
  * @manager: This class instance
@@ -645,15 +540,11 @@ gpm_manager_get_low_battery (GpmManager *manager, gboolean *low_battery, GError
 static void
 idle_do_sleep (GpmManager *manager)
 {
-	gboolean on_ac;
 	gchar *action = NULL;
 	gboolean ret;
 	GError *error = NULL;
 
-	/* find if we are on AC power */
-	on_ac = gpm_ac_adapter_is_present (manager->priv->ac_adapter);
-
-	if (on_ac)
+	if (!manager->priv->on_battery)
 		action = gconf_client_get_string (manager->priv->conf, GPM_CONF_ACTIONS_SLEEP_TYPE_AC, NULL);
 	else
 		action = gconf_client_get_string (manager->priv->conf, GPM_CONF_ACTIONS_SLEEP_TYPE_BATT, NULL);
@@ -717,13 +608,12 @@ idle_changed_cb (GpmIdle *idle, GpmIdleMode mode, GpmManager *manager)
 		return;
 	}
 
-	/* Ignore timeout events when the lid is closed, as the DPMS is
-	 * already off, and we don't want to perform policy actions or re-enable
-	 * the screen when the user moves the mouse on systems that do not
-	 * support hardware blanking.
-	 * Details are here: https://launchpad.net/malone/bugs/22522 */
-	if (gpm_button_is_lid_closed (manager->priv->button)) {
-		egg_debug ("lid is closed, so we are ignoring idle state changes");
+	/* Ignore back-to-NORMAL events when the lid is closed, as the DPMS is
+	 * already off, and we don't want to re-enable the screen when the user
+	 * moves the mouse on systems that do not support hardware blanking. */
+	if (gpm_button_is_lid_closed (manager->priv->button) &&
+	    mode == GPM_IDLE_MODE_NORMAL) {
+		egg_debug ("lid is closed, so we are ignoring ->NORMAL state changes");
 		return;
 	}
 
@@ -752,13 +642,6 @@ idle_changed_cb (GpmIdle *idle, GpmIdleMode mode, GpmManager *manager)
 static void
 lid_button_pressed (GpmManager *manager, gboolean pressed)
 {
-	gboolean on_ac;
-	gboolean has_inhibit;
-	gboolean do_policy;
-	gchar *action;
-
-	on_ac = gpm_ac_adapter_is_present (manager->priv->ac_adapter);
-
 	if (pressed)
 		gpm_manager_play (manager, GPM_MANAGER_SOUND_LID_CLOSE, FALSE);
 	else
@@ -770,36 +653,13 @@ lid_button_pressed (GpmManager *manager, gboolean pressed)
 		return;
 	}
 
-	if (on_ac) {
+	if (!manager->priv->on_battery) {
 		egg_debug ("Performing AC policy");
 		manager_policy_do (manager, GPM_CONF_BUTTON_LID_AC,
 				   _("The lid has been closed on ac power."));
 		return;
 	}
 
-	/* default */
-	do_policy = TRUE;
-
-	/* are we inhibited? */
-	gpm_inhibit_has_inhibit (manager->priv->inhibit, &has_inhibit, NULL);
-
-	/* do not do lid close action if suspend (or hibernate) */
-	if (has_inhibit) {
-		/* get the policy action for battery */
-		action = gconf_client_get_string (manager->priv->conf, GPM_CONF_BUTTON_LID_BATT, NULL);
-
-		/* if we are trying to suspend or hibernate then don't do action */
-		if ((strcmp (action, ACTION_SUSPEND) == 0) ||
-		    (strcmp (action, ACTION_HIBERNATE) == 0))
-			do_policy = FALSE;
-		g_free (action);
-	}
-
-	if (do_policy == FALSE) {
-		egg_debug ("Not doing lid policy action as inhibited as set to sleep");
-		return;
-	}
-
 	egg_debug ("Performing battery policy");
 	manager_policy_do (manager, GPM_CONF_BUTTON_LID_BATT,
 			   _("The lid has been closed on battery power."));
@@ -827,11 +687,11 @@ update_dpms_throttle (GpmManager *manager)
 }
 
 static void
-update_ac_throttle (GpmManager *manager, gboolean on_ac)
+update_ac_throttle (GpmManager *manager)
 {
 	/* Throttle the manager when we are not on AC power so we don't
 	   waste the battery */
-	if (on_ac) {
+	if (!manager->priv->on_battery) {
 		if (manager->priv->screensaver_ac_throttle_id != 0) {
 			gpm_screensaver_remove_throttle (manager->priv->screensaver, manager->priv->screensaver_ac_throttle_id);
 			manager->priv->screensaver_ac_throttle_id = 0;
@@ -923,19 +783,24 @@ button_pressed_cb (GpmButton *button, const gchar *type, GpmManager *manager)
 }
 
 /**
- * power_on_ac_changed_cb:
- * @power: The power class instance
- * @on_ac: if we are on AC power
- * @manager: This class instance
- *
- * Does the actions when the ac power source is inserted/removed.
+ * gpm_manager_client_changed_cb:
  **/
 static void
-ac_adapter_changed_cb (GpmAcAdapter *ac_adapter, gboolean on_ac, GpmManager *manager)
+gpm_manager_client_changed_cb (DkpClient *client, GpmManager *manager)
 {
 	gboolean event_when_closed;
-	gboolean power_save;
 	guint brightness;
+	gboolean on_battery;
+
+	/* get the on-battery state */
+	on_battery = dkp_client_on_battery (manager->priv->client);
+	if (on_battery == manager->priv->on_battery) {
+		egg_debug ("same state as before, ignoring");
+		return;
+	}
+
+	/* save in local cache */
+	manager->priv->on_battery = on_battery;
 
 	/* ConsoleKit says we are not on active console */
 	if (!egg_console_kit_is_active (manager->priv->console)) {
@@ -943,9 +808,9 @@ ac_adapter_changed_cb (GpmAcAdapter *ac_adapter, gboolean on_ac, GpmManager *man
 		return;
 	}
 
-	egg_debug ("Setting on-ac: %d", on_ac);
+	egg_debug ("on_battery: %d", on_battery);
 
-	if (on_ac)
+	if (!on_battery)
 		brightness = gconf_client_get_int (manager->priv->conf, GPM_CONF_KEYBOARD_BRIGHTNESS_AC, NULL);
 	else
 		brightness = gconf_client_get_int (manager->priv->conf, GPM_CONF_KEYBOARD_BRIGHTNESS_BATT, NULL);
@@ -953,37 +818,25 @@ ac_adapter_changed_cb (GpmAcAdapter *ac_adapter, gboolean on_ac, GpmManager *man
 
 	gpm_manager_sync_policy_sleep (manager);
 
-	update_ac_throttle (manager, on_ac);
+	update_ac_throttle (manager);
 
 	/* simulate user input, but only when the lid is open */
-	if (gpm_button_is_lid_closed (manager->priv->button) == FALSE)
+	if (!gpm_button_is_lid_closed (manager->priv->button))
 		gpm_screensaver_poke (manager->priv->screensaver);
 
-	if (on_ac)
+	if (!on_battery)
 		gpm_manager_play (manager, GPM_MANAGER_SOUND_POWER_PLUG, FALSE);
 	else
 		gpm_manager_play (manager, GPM_MANAGER_SOUND_POWER_UNPLUG, FALSE);
 
-	egg_debug ("emitting on-ac-changed : %i", on_ac);
-	g_signal_emit (manager, signals [ON_BATTERY_CHANGED], 0, !on_ac);
-
-	on_ac = gpm_ac_adapter_is_present (manager->priv->ac_adapter);
-	if (on_ac)
-		power_save = gconf_client_get_bool (manager->priv->conf, GPM_CONF_LOWPOWER_AC, NULL);
-	else
-		power_save = gconf_client_get_bool (manager->priv->conf, GPM_CONF_LOWPOWER_BATT, NULL);
-	if (manager->priv->low_power != power_save)
-		g_signal_emit (manager, signals [POWER_SAVE_STATUS_CHANGED], 0, power_save);
-	manager->priv->low_power = power_save;
-
-	/* We do the lid close on battery action if the ac_adapter is removed
+	/* We do the lid close on battery action if the ac adapter is removed
 	   when the laptop is closed and on battery. Fixes #331655 */
 	event_when_closed = gconf_client_get_bool (manager->priv->conf, GPM_CONF_ACTIONS_SLEEP_WHEN_CLOSED, NULL);
 
 	/* We keep track of the lid state so we can do the
-	   lid close on battery action if the ac_adapter is removed when the laptop
+	   lid close on battery action if the ac adapter is removed when the laptop
 	   is closed. Fixes #331655 */
-	if (event_when_closed && on_ac == FALSE && gpm_button_is_lid_closed (manager->priv->button)) {
+	if (event_when_closed && on_battery && gpm_button_is_lid_closed (manager->priv->button)) {
 		manager_policy_do (manager, GPM_CONF_BUTTON_LID_BATT,
 				   _("The lid has been closed, and the ac adapter "
 				     "removed (and gconf is okay)."));
@@ -1014,28 +867,7 @@ static void
 gpm_manager_class_init (GpmManagerClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-
 	object_class->finalize = gpm_manager_finalize;
-
-	signals [ON_BATTERY_CHANGED] =
-		g_signal_new ("on-battery-changed",
-			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmManagerClass, on_battery_changed),
-			      NULL, NULL, g_cclosure_marshal_VOID__BOOLEAN,
-			      G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
-	signals [LOW_BATTERY_CHANGED] =
-		g_signal_new ("low-battery-changed",
-			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmManagerClass, low_battery_changed),
-			      NULL, NULL, g_cclosure_marshal_VOID__BOOLEAN,
-			      G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
-	signals [POWER_SAVE_STATUS_CHANGED] =
-		g_signal_new ("power-save-status-changed",
-			      G_TYPE_FROM_CLASS (object_class), G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (GpmManagerClass, power_save_status_changed),
-			      NULL, NULL, g_cclosure_marshal_VOID__BOOLEAN,
-			      G_TYPE_NONE, 1, G_TYPE_BOOLEAN);
-
 	g_type_class_add_private (klass, sizeof (GpmManagerPrivate));
 }
 
@@ -1049,7 +881,6 @@ gpm_conf_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *e
 {
 	GConfValue *value;
 	gint brightness;
-	gboolean on_ac;
 
 	value = gconf_entry_get_value (entry);
 	if (value == NULL)
@@ -1062,17 +893,16 @@ gpm_conf_gconf_key_changed_cb (GConfClient *client, guint cnxn_id, GConfEntry *e
 		gpm_manager_sync_policy_sleep (manager);
 
 	/* set keyboard brightness */
-	on_ac = gpm_ac_adapter_is_present (manager->priv->ac_adapter);
-	if (strcmp (entry->key, GPM_CONF_KEYBOARD_BRIGHTNESS_AC) == 0) {
+	if (strcmp (entry->key, GPM_CONF_KEYBOARD_BRIGHTNESS_BATT) == 0) {
 
 		brightness = gconf_value_get_int (value);
-		if (on_ac)
+		if (manager->priv->on_battery)
 			gpm_brightness_kbd_set_std (manager->priv->brightness_kbd, brightness);
 
-	} else if (strcmp (entry->key, GPM_CONF_KEYBOARD_BRIGHTNESS_BATT) == 0) {
+	} else if (strcmp (entry->key, GPM_CONF_KEYBOARD_BRIGHTNESS_AC) == 0) {
 
 		brightness = gconf_client_get_int (manager->priv->conf, GPM_CONF_KEYBOARD_BRIGHTNESS_AC, NULL);
-		if (on_ac == FALSE)
+		if (!manager->priv->on_battery)
 			gpm_brightness_kbd_set_std (manager->priv->brightness_kbd, brightness);
 
 	}
@@ -1155,7 +985,7 @@ gpm_manager_perhaps_recall (GpmManager *manager)
  * gpm_engine_perhaps_recall_cb:
  */
 static void
-gpm_engine_perhaps_recall_cb (GpmEngine *engine, DkpDeviceType type, gchar *oem_vendor, gchar *website, GpmManager *manager)
+gpm_engine_perhaps_recall_cb (GpmEngine *engine, DkpDevice *device, gchar *oem_vendor, gchar *website, GpmManager *manager)
 {
 	g_object_set_data (G_OBJECT (manager), "recall-oem-vendor", (gpointer) g_strdup (oem_vendor));
 	g_object_set_data (G_OBJECT (manager), "recall-oem-website", (gpointer) g_strdup (website));
@@ -1163,18 +993,6 @@ gpm_engine_perhaps_recall_cb (GpmEngine *engine, DkpDeviceType type, gchar *oem_
 	g_timeout_add_seconds (GPM_MANAGER_RECALL_DELAY, (GSourceFunc) gpm_manager_perhaps_recall, manager);
 }
 
-/**
- * gpm_engine_low_capacity_cb:
- */
-static void
-gpm_engine_low_capacity_cb (GpmEngine *engine, DkpDeviceType type, guint capacity, GpmManager *manager)
-{
-	/* We should notify the user if the battery has a low capacity,
-	 * where capacity is the ratio of the last_full capacity with that of
-	 * the design capacity. (#326740) */
-	gpm_notify_low_capacity (manager->priv->notify, capacity);
-}
-
 /**
  * gpm_engine_icon_changed_cb:
  */
@@ -1197,18 +1015,67 @@ gpm_engine_summary_changed_cb (GpmEngine *engine, gchar *summary, GpmManager *ma
  * gpm_engine_low_capacity_cb:
  */
 static void
-gpm_engine_fully_charged_cb (GpmEngine *engine, DkpDeviceType type, GpmManager *manager)
+gpm_engine_low_capacity_cb (GpmEngine *engine, DkpDevice *device, GpmManager *manager)
 {
+	gdouble capacity;
+
+	/* get device properties */
+	g_object_get (device,
+		      "capacity", &capacity,
+		      NULL);
+
+	/* We should notify the user if the battery has a low capacity,
+	 * where capacity is the ratio of the last_full capacity with that of
+	 * the design capacity. (#326740) */
+	gpm_notify_low_capacity (manager->priv->notify, capacity);
+}
+
+/**
+ * gpm_engine_fully_charged_cb:
+ */
+static void
+gpm_engine_fully_charged_cb (GpmEngine *engine, DkpDevice *device, GpmManager *manager)
+{
+	DkpDeviceType type;
+	gboolean ret;
+
+	/* only action this if specified in gconf */
+	ret = gconf_client_get_bool (manager->priv->conf, GPM_CONF_NOTIFY_FULLY_CHARGED, NULL);
+	if (!ret) {
+		egg_debug ("no notification");
+		return;
+	}
+
+	/* get device properties */
+	g_object_get (device,
+		      "type", &type,
+		      NULL);
+
 	if (type == DKP_DEVICE_TYPE_BATTERY)
 		gpm_notify_fully_charged_primary (manager->priv->notify);
 }
 
 /**
- * gpm_engine_low_capacity_cb:
+ * gpm_engine_discharging_cb:
  */
 static void
-gpm_engine_discharging_cb (GpmEngine *engine, DkpDeviceType type, GpmManager *manager)
+gpm_engine_discharging_cb (GpmEngine *engine, DkpDevice *device, GpmManager *manager)
 {
+	DkpDeviceType type;
+	gboolean ret;
+
+	/* only action this if specified in gconf */
+	ret = gconf_client_get_bool (manager->priv->conf, GPM_CONF_NOTIFY_DISCHARGING, NULL);
+	if (!ret) {
+		egg_debug ("no notification");
+		return;
+	}
+
+	/* get device properties */
+	g_object_get (device,
+		      "type", &type,
+		      NULL);
+
 	if (type == DKP_DEVICE_TYPE_BATTERY)
 		gpm_notify_discharging_primary (manager->priv->notify);
 	else if (type == DKP_DEVICE_TYPE_UPS)
@@ -1245,44 +1112,63 @@ control_sleep_failure_cb (GpmControl *control, GpmControlAction action, GpmManag
  * gpm_engine_charge_low_cb:
  */
 static void
-gpm_engine_charge_low_cb (GpmEngine *engine, DkpObject *obj, GpmManager *manager)
+gpm_engine_charge_low_cb (GpmEngine *engine, DkpDevice *device, GpmManager *manager)
 {
 	const gchar *title = NULL;
 	gchar *message = NULL;
 	gchar *remaining_text;
-	gchar *icon;
+	gchar *icon = NULL;
+	DkpDeviceType type;
+	gdouble percentage;
+	gint64 time_to_empty;
 
-	if (obj->type == DKP_DEVICE_TYPE_BATTERY) {
+	/* get device properties */
+	g_object_get (device,
+		      "type", &type,
+		      "percentage", &percentage,
+		      "time-to-empty", &time_to_empty,
+		      NULL);
+
+	/* check to see if the batteries have not noticed we are on AC */
+	if (type == DKP_DEVICE_TYPE_BATTERY) {
+		if (!manager->priv->on_battery) {
+			egg_warning ("ignoring critically low message as we are not on battery power");
+			goto out;
+		}
+	}
+
+	if (type == DKP_DEVICE_TYPE_BATTERY) {
 		title = _("Laptop battery low");
-		remaining_text = gpm_get_timestring (obj->time_to_empty);
+		remaining_text = gpm_get_timestring (time_to_empty);
 		message = g_strdup_printf (_("You have approximately <b>%s</b> of remaining battery life (%.1f%%)"),
-					   remaining_text, obj->percentage);
-	} else if (obj->type == DKP_DEVICE_TYPE_UPS) {
+					   remaining_text, percentage);
+	} else if (type == DKP_DEVICE_TYPE_UPS) {
 		title = _("UPS low");
-		remaining_text = gpm_get_timestring (obj->time_to_empty);
+		remaining_text = gpm_get_timestring (time_to_empty);
 		message = g_strdup_printf (_("You have approximately <b>%s</b> of remaining UPS backup power (%.1f%%)"),
-					   remaining_text, obj->percentage);
-	} else if (obj->type == DKP_DEVICE_TYPE_MOUSE) {
+					   remaining_text, percentage);
+	} else if (type == DKP_DEVICE_TYPE_MOUSE) {
 		title = _("Mouse battery low");
-		message = g_strdup_printf (_("The wireless mouse attached to this computer is low in power (%.1f%%)"), obj->percentage);
+		message = g_strdup_printf (_("The wireless mouse attached to this computer is low in power (%.1f%%)"), percentage);
 
-	} else if (obj->type == DKP_DEVICE_TYPE_KEYBOARD) {
+	} else if (type == DKP_DEVICE_TYPE_KEYBOARD) {
 		title = _("Keyboard battery low");
-		message = g_strdup_printf (_("The wireless keyboard attached to this computer is low in power (%.1f%%)"), obj->percentage);
+		message = g_strdup_printf (_("The wireless keyboard attached to this computer is low in power (%.1f%%)"), percentage);
 
-	} else if (obj->type == DKP_DEVICE_TYPE_PDA) {
+	} else if (type == DKP_DEVICE_TYPE_PDA) {
 		title = _("PDA battery low");
-		message = g_strdup_printf (_("The PDA attached to this computer is low in power (%.1f%%)"), obj->percentage);
+		message = g_strdup_printf (_("The PDA attached to this computer is low in power (%.1f%%)"), percentage);
 
-	} else if (obj->type == DKP_DEVICE_TYPE_PHONE) {
+	} else if (type == DKP_DEVICE_TYPE_PHONE) {
 		title = _("Cell phone battery low");
-		message = g_strdup_printf (_("The cell phone attached to this computer is low in power (%.1f%%)"), obj->percentage);
+		message = g_strdup_printf (_("The cell phone attached to this computer is low in power (%.1f%%)"), percentage);
 	}
 
 	/* get correct icon */
-	icon = gpm_devicekit_get_object_icon (obj);
+	icon = gpm_devicekit_get_object_icon (device);
 	gpm_notify_display (manager->priv->notify, title, message, GPM_NOTIFY_TIMEOUT_LONG, icon, GPM_NOTIFY_URGENCY_NORMAL);
 	gpm_manager_play (manager, GPM_MANAGER_SOUND_BATTERY_LOW, TRUE);
+out:
 	g_free (icon);
 	g_free (message);
 }
@@ -1301,19 +1187,37 @@ gpm_manager_get_time_until_action_text (GpmManager *manager)
  * gpm_engine_charge_critical_cb:
  */
 static void
-gpm_engine_charge_critical_cb (GpmEngine *engine, DkpObject *obj, GpmManager *manager)
+gpm_engine_charge_critical_cb (GpmEngine *engine, DkpDevice *device, GpmManager *manager)
 {
 	const gchar *title = NULL;
 	gchar *message = NULL;
 	gchar *action_text = NULL;
 	gchar *remaining_text;
 	gchar *action;
-	gchar *icon;
+	gchar *icon = NULL;
 	gchar *time_text;
+	DkpDeviceType type;
+	gdouble percentage;
+	gint64 time_to_empty;
 
-	if (obj->type == DKP_DEVICE_TYPE_BATTERY) {
+	/* get device properties */
+	g_object_get (device,
+		      "type", &type,
+		      "percentage", &percentage,
+		      "time-to-empty", &time_to_empty,
+		      NULL);
+
+	/* check to see if the batteries have not noticed we are on AC */
+	if (type == DKP_DEVICE_TYPE_BATTERY) {
+		if (!manager->priv->on_battery) {
+			egg_warning ("ignoring critically low message as we are not on battery power");
+			goto out;
+		}
+	}
+
+	if (type == DKP_DEVICE_TYPE_BATTERY) {
 		title = _("Laptop battery critically low");
-		remaining_text = gpm_get_timestring (obj->time_to_empty);
+		remaining_text = gpm_get_timestring (time_to_empty);
 		time_text = gpm_manager_get_time_until_action_text (manager);
 
 		/* we have to do different warnings depending on the policy */
@@ -1334,45 +1238,46 @@ gpm_engine_charge_critical_cb (GpmEngine *engine, DkpObject *obj, GpmManager *ma
 			action_text = g_strdup_printf (_("This computer will shutdown in %s if the AC is not connected."), time_text);
 
 		message = g_strdup_printf (_("You have approximately <b>%s</b> of remaining battery life (%.1f%%). %s"),
-					   remaining_text, obj->percentage, action_text);
+					   remaining_text, percentage, action_text);
 
 		g_free (action);
 		g_free (action_text);
 		g_free (remaining_text);
 		g_free (time_text);
-	} else if (obj->type == DKP_DEVICE_TYPE_UPS) {
+	} else if (type == DKP_DEVICE_TYPE_UPS) {
 		title = _("UPS critically low");
-		remaining_text = gpm_get_timestring (obj->time_to_empty);
+		remaining_text = gpm_get_timestring (time_to_empty);
 		message = g_strdup_printf (_("You have approximately <b>%s</b> of remaining UPS power (%.1f%%). "
 					     "Restore AC power to your computer to avoid losing data."),
-					   remaining_text, obj->percentage);
+					   remaining_text, percentage);
 		g_free (remaining_text);
-	} else if (obj->type == DKP_DEVICE_TYPE_MOUSE) {
+	} else if (type == DKP_DEVICE_TYPE_MOUSE) {
 		title = _("Mouse battery low");
 		message = g_strdup_printf (_("The wireless mouse attached to this computer is very low in power (%.1f%%). "
 					     "This device will soon stop functioning if not charged."),
-					   obj->percentage);
-	} else if (obj->type == DKP_DEVICE_TYPE_KEYBOARD) {
+					   percentage);
+	} else if (type == DKP_DEVICE_TYPE_KEYBOARD) {
 		title = _("Keyboard battery low");
 		message = g_strdup_printf (_("The wireless keyboard attached to this computer is very low in power (%.1f%%). "
 					     "This device will soon stop functioning if not charged."),
-					   obj->percentage);
-	} else if (obj->type == DKP_DEVICE_TYPE_PDA) {
+					   percentage);
+	} else if (type == DKP_DEVICE_TYPE_PDA) {
 		title = _("PDA battery low");
 		message = g_strdup_printf (_("The PDA attached to this computer is very low in power (%.1f%%). "
 					     "This device will soon stop functioning if not charged."),
-					   obj->percentage);
-	} else if (obj->type == DKP_DEVICE_TYPE_PHONE) {
+					   percentage);
+	} else if (type == DKP_DEVICE_TYPE_PHONE) {
 		title = _("Cell phone battery low");
 		message = g_strdup_printf (_("Your cell phone is very low in power (%.1f%%). "
 					     "This device will soon stop functioning if not charged."),
-					   obj->percentage);
+					   percentage);
 	}
 
 	/* get correct icon */
-	icon = gpm_devicekit_get_object_icon (obj);
+	icon = gpm_devicekit_get_object_icon (device);
 	gpm_notify_display (manager->priv->notify, title, message, GPM_NOTIFY_TIMEOUT_LONG, icon, GPM_NOTIFY_URGENCY_CRITICAL);
 	gpm_manager_play (manager, GPM_MANAGER_SOUND_BATTERY_LOW, TRUE);
+out:
 	g_free (icon);
 	g_free (message);
 }
@@ -1381,14 +1286,28 @@ gpm_engine_charge_critical_cb (GpmEngine *engine, DkpObject *obj, GpmManager *ma
  * gpm_engine_charge_action_cb:
  */
 static void
-gpm_engine_charge_action_cb (GpmEngine *engine, DkpObject *obj, GpmManager *manager)
+gpm_engine_charge_action_cb (GpmEngine *engine, DkpDevice *device, GpmManager *manager)
 {
 	const gchar *title = NULL;
 	gchar *action;
 	gchar *message = NULL;
-	gchar *icon;
+	gchar *icon = NULL;
+	DkpDeviceType type;
 
-	if (obj->type == DKP_DEVICE_TYPE_BATTERY) {
+	/* get device properties */
+	g_object_get (device,
+		      "type", &type,
+		      NULL);
+
+	/* check to see if the batteries have not noticed we are on AC */
+	if (type == DKP_DEVICE_TYPE_BATTERY) {
+		if (!manager->priv->on_battery) {
+			egg_warning ("ignoring critically low message as we are not on battery power");
+			goto out;
+		}
+	}
+
+	if (type == DKP_DEVICE_TYPE_BATTERY) {
 		title = _("Laptop battery critically low");
 
 		/* we have to do different warnings depending on the policy */
@@ -1420,7 +1339,7 @@ gpm_engine_charge_action_cb (GpmEngine *engine, DkpObject *obj, GpmManager *mana
 		/* wait 10 seconds for user-panic */
 		g_timeout_add_seconds (10, (GSourceFunc) manager_critical_action_do, manager);
 
-	} else if (obj->type == DKP_DEVICE_TYPE_UPS) {
+	} else if (type == DKP_DEVICE_TYPE_UPS) {
 		title = _("UPS critically low");
 
 		/* we have to do different warnings depending on the policy */
@@ -1449,55 +1368,16 @@ gpm_engine_charge_action_cb (GpmEngine *engine, DkpObject *obj, GpmManager *mana
 		return;
 
 	/* get correct icon */
-	icon = gpm_devicekit_get_object_icon (obj);
+	icon = gpm_devicekit_get_object_icon (device);
 	gpm_notify_display (manager->priv->notify,
 			    title, message, GPM_NOTIFY_TIMEOUT_LONG,
 			    icon, GPM_NOTIFY_URGENCY_CRITICAL);
 	gpm_manager_play (manager, GPM_MANAGER_SOUND_BATTERY_LOW, TRUE);
+out:
 	g_free (icon);
 	g_free (message);
 }
 
-/**
- * has_inhibit_changed_cb:
- **/
-static void
-has_inhibit_changed_cb (GpmInhibit *inhibit, gboolean has_inhibit, GpmManager *manager)
-{
-	gboolean is_laptop = FALSE;
-	gboolean show_inhibit_lid;
-	gchar *action = NULL;
-
-	/* we don't care about uninhibits */
-	if (has_inhibit == FALSE)
-		return;
-
-	/* only show this if specified in gconf */
-	show_inhibit_lid = gconf_client_get_bool (manager->priv->conf, GPM_CONF_NOTIFY_INHIBIT_LID, NULL);
-
-	/* we've already shown the UI and been clicked */
-	if (show_inhibit_lid == FALSE)
-		return;
-
-/* FIXME: detect if laptop */
-
-	/* we don't warn for desktops, as they do not have a lid... */
-	if (is_laptop == FALSE)
-		return;
-
-	/* get the policy action for battery */
-	action = gconf_client_get_string (manager->priv->conf, GPM_CONF_BUTTON_LID_BATT, NULL);
-	if (action == NULL)
-		return;
-
-	/* if the policy on lid close is sleep then show a warning */
-	if ((strcmp (action, ACTION_SUSPEND) == 0) ||
-	    (strcmp (action, ACTION_HIBERNATE) == 0))
-		gpm_notify_inhibit_lid (manager->priv->notify);
-
-	g_free (action);
-}
-
 /**
  * brightness_kbd_changed_cb:
  * @brightness: The GpmBrightnessKbd class instance
@@ -1543,8 +1423,30 @@ dpms_mode_changed_cb (GpmDpms *dpms, GpmDpmsMode mode, GpmManager *manager)
 static void
 gpm_manager_console_kit_active_changed_cb (EggConsoleKit *console, gboolean active, GpmManager *manager)
 {
+	gboolean ret;
+
 	egg_debug ("console now %s", active ? "active" : "inactive");
-	/* FIXME: do we need to do policy actions when we switch? */
+
+	/* gone inactive */
+	if (!active)
+		return;
+
+	/* get lid state */
+	ret = gpm_button_is_lid_closed (manager->priv->button);
+	if (!ret)
+		return;
+
+	/* get ac state */
+	if (!manager->priv->on_battery) {
+		egg_debug ("Performing AC policy as become active when lid down");
+		manager_policy_do (manager, GPM_CONF_BUTTON_LID_AC,
+				   _("The lid has been found closed on ac power."));
+		return;
+	}
+
+	egg_debug ("Performing battery policy as become active when lid down");
+	manager_policy_do (manager, GPM_CONF_BUTTON_LID_BATT,
+			   _("The lid has been found closed on battery power."));
 }
 
 /**
@@ -1557,7 +1459,6 @@ gpm_manager_init (GpmManager *manager)
 	gboolean check_type_cpu;
 	DBusGConnection *connection;
 	GError *error = NULL;
-	gboolean on_ac;
 	guint version;
 
 	manager->priv = GPM_MANAGER_GET_PRIVATE (manager);
@@ -1578,10 +1479,13 @@ gpm_manager_init (GpmManager *manager)
 
 	manager->priv->notify = gpm_notify_new ();
 	manager->priv->conf = gconf_client_get_default ();
+	manager->priv->client = dkp_client_new ();
+	g_signal_connect (manager->priv->client, "changed",
+			  G_CALLBACK (gpm_manager_client_changed_cb), manager);
 
 	/* watch gnome-power-manager keys */
 	gconf_client_add_dir (manager->priv->conf, GPM_CONF_DIR,
-			      GCONF_CLIENT_PRELOAD_NONE, NULL);
+			      GCONF_CLIENT_PRELOAD_RECURSIVE, NULL);
 	gconf_client_notify_add (manager->priv->conf, GPM_CONF_DIR,
 				 (GConfClientNotifyFunc) gpm_conf_gconf_key_changed_cb,
 				 manager, NULL, NULL);
@@ -1599,17 +1503,8 @@ gpm_manager_init (GpmManager *manager)
 		egg_error ("no gconf schema installed!");
 	}
 
-	/* we use ac_adapter so we can poke the screensaver and throttle */
-	manager->priv->ac_adapter = gpm_ac_adapter_new ();
-	g_signal_connect (manager->priv->ac_adapter, "ac-adapter-changed",
-			  G_CALLBACK (ac_adapter_changed_cb), manager);
-
 	/* coldplug so we are in the correct state at startup */
-	on_ac = gpm_ac_adapter_is_present (manager->priv->ac_adapter);
-	if (on_ac)
-		manager->priv->low_power = gconf_client_get_bool (manager->priv->conf, GPM_CONF_LOWPOWER_AC, NULL);
-	else
-		manager->priv->low_power = gconf_client_get_bool (manager->priv->conf, GPM_CONF_LOWPOWER_BATT, NULL);
+	manager->priv->on_battery = dkp_client_on_battery (manager->priv->client);
 
 	manager->priv->button = gpm_button_new ();
 	g_signal_connect (manager->priv->button, "button-pressed",
@@ -1650,16 +1545,6 @@ gpm_manager_init (GpmManager *manager)
 	g_signal_connect (manager->priv->dpms, "mode-changed",
 			  G_CALLBACK (dpms_mode_changed_cb), manager);
 
-	/* use a class to handle the complex stuff */
-	egg_debug ("creating new inhibit instance");
-	manager->priv->inhibit = gpm_inhibit_new ();
-	g_signal_connect (manager->priv->inhibit, "has-inhibit-changed",
-			  G_CALLBACK (has_inhibit_changed_cb), manager);
-	/* add the interface */
-	dbus_g_object_type_install_info (GPM_TYPE_INHIBIT, &dbus_glib_gpm_inhibit_object_info);
-	dbus_g_connection_register_g_object (connection, GPM_DBUS_PATH_INHIBIT,
-					     G_OBJECT (manager->priv->inhibit));
-
 	/* use the control object */
 	egg_debug ("creating new control instance");
 	manager->priv->control = gpm_control_new ();
@@ -1698,8 +1583,7 @@ gpm_manager_init (GpmManager *manager)
 			  G_CALLBACK (gpm_engine_charge_action_cb), manager);
 
 	/* update ac throttle */
-	on_ac = gpm_ac_adapter_is_present (manager->priv->ac_adapter);
-	update_ac_throttle (manager, on_ac);
+	update_ac_throttle (manager);
 }
 
 /**
@@ -1725,7 +1609,6 @@ gpm_manager_finalize (GObject *object)
 	g_object_unref (manager->priv->idle);
 	g_object_unref (manager->priv->engine);
 	g_object_unref (manager->priv->tray_icon);
-	g_object_unref (manager->priv->inhibit);
 	g_object_unref (manager->priv->screensaver);
 	g_object_unref (manager->priv->notify);
 	g_object_unref (manager->priv->prefs_server);
@@ -1735,6 +1618,7 @@ gpm_manager_finalize (GObject *object)
 	g_object_unref (manager->priv->brightness_kbd);
 	g_object_unref (manager->priv->backlight);
 	g_object_unref (manager->priv->console);
+	g_object_unref (manager->priv->client);
 
 	G_OBJECT_CLASS (gpm_manager_parent_class)->finalize (object);
 }
