@@ -26,15 +26,13 @@
 #include <glib.h>
 #include <glib/gi18n.h>
 
-#include <glade/glade.h>
 #include <gtk/gtk.h>
 #include <dbus/dbus-glib.h>
 #include <gconf/gconf-client.h>
+#include <devkit-power-gobject/devicekit-power.h>
 
 #include "egg-debug.h"
 #include "egg-color.h"
-#include "egg-string.h"
-#include "egg-obj-list.h"
 #include "egg-array-float.h"
 #include "egg-unique.h"
 
@@ -42,13 +40,8 @@
 #include "gpm-stock-icons.h"
 #include "gpm-devicekit.h"
 #include "gpm-graph-widget.h"
-#include "dkp-client.h"
-#include "dkp-history-obj.h"
-#include "dkp-stats-obj.h"
-#include "dkp-device.h"
-#include "dkp-wakeups.h"
 
-static GladeXML *glade_xml = NULL;
+static GtkBuilder *builder = NULL;
 static GtkListStore *list_store_info = NULL;
 static GtkListStore *list_store_devices = NULL;
 static GtkListStore *list_store_wakeups = NULL;
@@ -59,6 +52,8 @@ static guint history_time;
 static GConfClient *gconf_client;
 static gfloat sigma_smoothing = 0.0f;
 static DkpWakeups *wakeups = NULL;
+static GtkWidget *graph_history = NULL;
+static GtkWidget *graph_statistics = NULL;
 
 enum {
 	GPM_INFO_COLUMN_TEXT,
@@ -230,13 +225,13 @@ gpm_stats_add_info_data (const gchar *attr, const gchar *text)
 /**
  * gpm_stats_update_smooth_data:
  **/
-static EggObjList *
-gpm_stats_update_smooth_data (EggObjList *list)
+static GPtrArray *
+gpm_stats_update_smooth_data (GPtrArray *list)
 {
 	guint i;
 	GpmPointObj *point;
-	GpmPointObj point_new;
-	EggObjList *new;
+	GpmPointObj *point_new;
+	GPtrArray *new;
 	EggArrayFloat *raw;
 	EggArrayFloat *convolved;
 	EggArrayFloat *outliers;
@@ -246,7 +241,7 @@ gpm_stats_update_smooth_data (EggObjList *list)
 	raw = egg_array_float_new (list->len);
 	convolved = egg_array_float_new (list->len);
 	for (i=0; i<list->len; i++) {
-		point = (GpmPointObj *) egg_obj_list_index (list, i);
+		point = (GpmPointObj *) g_ptr_array_index (list, i);
 		egg_array_float_set (raw, i, point->y);
 	}
 
@@ -258,15 +253,14 @@ gpm_stats_update_smooth_data (EggObjList *list)
 	convolved = egg_array_float_convolve (outliers, gaussian);
 
 	/* add the smoothed data back into a new array */
-	new = egg_obj_list_new ();
-	egg_obj_list_set_copy (new, (EggObjListCopyFunc) dkp_point_obj_copy);
-	egg_obj_list_set_free (new, (EggObjListFreeFunc) dkp_point_obj_free);
+	new = g_ptr_array_new ();
 	for (i=0; i<list->len; i++) {
-		point = (GpmPointObj *) egg_obj_list_index (list, i);
-		point_new.color = point->color;
-		point_new.x = point->x;
-		point_new.y = egg_array_float_get (convolved, i);
-		egg_obj_list_add (new, &point_new);
+		point = (GpmPointObj *) g_ptr_array_index (list, i);
+		point_new = g_new0 (GpmPointObj, 1);
+		point_new->color = point->color;
+		point_new->x = point->x;
+		point_new->y = egg_array_float_get (convolved, i);
+		g_ptr_array_add (new, point_new);
 	}
 
 	/* free data */
@@ -313,116 +307,165 @@ gpm_stats_bool_to_text (gboolean ret)
  * gpm_stats_update_info_page_details:
  **/
 static void
-gpm_stats_update_info_page_details (const DkpDevice *device)
+gpm_stats_update_info_page_details (DkpDevice *device)
 {
-	const DkpObject *obj;
 	struct tm *time_tm;
 	time_t t;
 	gchar time_buf[256];
 	gchar *text;
+	guint refreshed;
+	DkpDeviceType type;
+	DkpDeviceState state;
+	DkpDeviceTechnology technology;
+	gdouble percentage;
+	gdouble capacity;
+	gdouble energy;
+	gdouble energy_empty;
+	gdouble energy_full;
+	gdouble energy_full_design;
+	gdouble energy_rate;
+	gdouble voltage;
+	gboolean online;
+	gboolean is_present;
+	gboolean power_supply;
+	gboolean is_rechargeable;
+	guint64 update_time;
+	gint64 time_to_full;
+	gint64 time_to_empty;
+	gchar *vendor = NULL;
+	gchar *serial = NULL;
+	gchar *model = NULL;
 
 	gtk_list_store_clear (list_store_info);
 
-	obj = dkp_device_get_object (device);
+	/* get device properties */
+	g_object_get (device,
+		      "type", &type,
+		      "state", &state,
+		      "percentage", &percentage,
+		      "online", &online,
+		      "update_time", &update_time,
+		      "power_supply", &power_supply,
+		      "is_rechargeable", &is_rechargeable,
+		      "is-present", &is_present,
+		      "time-to-full", &time_to_full,
+		      "time-to-empty", &time_to_empty,
+		      "technology", &technology,
+		      "capacity", &capacity,
+		      "energy", &energy,
+		      "energy-empty", &energy_empty,
+		      "energy-full", &energy_full,
+		      "energy-full-design", &energy_full_design,
+		      "energy-rate", &energy_rate,
+		      "voltage", &voltage,
+		      "vendor", &vendor,
+		      "serial", &serial,
+		      "model", &model,
+		      NULL);
 
 	/* get a human readable time */
-	t = (time_t) obj->update_time;
+	t = (time_t) update_time;
 	time_tm = localtime (&t);
 	strftime (time_buf, sizeof time_buf, "%c", time_tm);
 
 	gpm_stats_add_info_data (_("Device"), dkp_device_get_object_path (device));
-	gpm_stats_add_info_data (_("Type"), gpm_device_type_to_localised_text (obj->type, 1));
-	if (!egg_strzero (obj->vendor))
-		gpm_stats_add_info_data (_("Vendor"), obj->vendor);
-	if (!egg_strzero (obj->model))
-		gpm_stats_add_info_data (_("Model"), obj->model);
-	if (!egg_strzero (obj->serial))
-		gpm_stats_add_info_data (_("Serial number"), obj->serial);
-	gpm_stats_add_info_data (_("Supply"), gpm_stats_bool_to_text (obj->power_supply));
+	gpm_stats_add_info_data (_("Type"), gpm_device_type_to_localised_text (type, 1));
+	if (vendor != NULL && vendor[0] != '\0')
+		gpm_stats_add_info_data (_("Vendor"), vendor);
+	if (model != NULL && model[0] != '\0')
+		gpm_stats_add_info_data (_("Model"), model);
+	if (serial != NULL && serial[0] != '\0')
+		gpm_stats_add_info_data (_("Serial number"), serial);
+	gpm_stats_add_info_data (_("Supply"), gpm_stats_bool_to_text (power_supply));
 
-	text = g_strdup_printf (_("%d seconds"), (int) (time (NULL) - obj->update_time));
+	refreshed = (int) (time (NULL) - update_time);
+	text = g_strdup_printf (ngettext ("%d second", "%d seconds", refreshed), refreshed);
 	gpm_stats_add_info_data (_("Refreshed"), text);
 	g_free (text);
 
-	if (obj->type == DKP_DEVICE_TYPE_BATTERY ||
-	    obj->type == DKP_DEVICE_TYPE_MOUSE ||
-	    obj->type == DKP_DEVICE_TYPE_KEYBOARD ||
-	    obj->type == DKP_DEVICE_TYPE_UPS)
-		gpm_stats_add_info_data (_("Present"), gpm_stats_bool_to_text (obj->is_present));
-	if (obj->type == DKP_DEVICE_TYPE_BATTERY ||
-	    obj->type == DKP_DEVICE_TYPE_MOUSE ||
-	    obj->type == DKP_DEVICE_TYPE_KEYBOARD)
-		gpm_stats_add_info_data (_("Rechargeable"), gpm_stats_bool_to_text (obj->is_rechargeable));
-	if (obj->type == DKP_DEVICE_TYPE_BATTERY ||
-	    obj->type == DKP_DEVICE_TYPE_MOUSE ||
-	    obj->type == DKP_DEVICE_TYPE_KEYBOARD)
-		gpm_stats_add_info_data (_("State"), dkp_device_state_to_text (obj->state));
-	if (obj->type == DKP_DEVICE_TYPE_BATTERY) {
-		text = g_strdup_printf ("%.1f Wh", obj->energy);
+	if (type == DKP_DEVICE_TYPE_BATTERY ||
+	    type == DKP_DEVICE_TYPE_MOUSE ||
+	    type == DKP_DEVICE_TYPE_KEYBOARD ||
+	    type == DKP_DEVICE_TYPE_UPS)
+		gpm_stats_add_info_data (_("Present"), gpm_stats_bool_to_text (is_present));
+	if (type == DKP_DEVICE_TYPE_BATTERY ||
+	    type == DKP_DEVICE_TYPE_MOUSE ||
+	    type == DKP_DEVICE_TYPE_KEYBOARD)
+		gpm_stats_add_info_data (_("Rechargeable"), gpm_stats_bool_to_text (is_rechargeable));
+	if (type == DKP_DEVICE_TYPE_BATTERY ||
+	    type == DKP_DEVICE_TYPE_MOUSE ||
+	    type == DKP_DEVICE_TYPE_KEYBOARD)
+		gpm_stats_add_info_data (_("State"), dkp_device_state_to_text (state));
+	if (type == DKP_DEVICE_TYPE_BATTERY) {
+		text = g_strdup_printf ("%.1f Wh", energy);
 		gpm_stats_add_info_data (_("Energy"), text);
 		g_free (text);
-		text = g_strdup_printf ("%.1f Wh", obj->energy_empty);
+		text = g_strdup_printf ("%.1f Wh", energy_empty);
 		gpm_stats_add_info_data (_("Energy when empty"), text);
 		g_free (text);
-		text = g_strdup_printf ("%.1f Wh", obj->energy_full);
+		text = g_strdup_printf ("%.1f Wh", energy_full);
 		gpm_stats_add_info_data (_("Energy when full"), text);
 		g_free (text);
-		text = g_strdup_printf ("%.1f Wh", obj->energy_full_design);
+		text = g_strdup_printf ("%.1f Wh", energy_full_design);
 		gpm_stats_add_info_data (_("Energy (design)"), text);
 		g_free (text);
 	}
-	if (obj->type == DKP_DEVICE_TYPE_BATTERY ||
-	    obj->type == DKP_DEVICE_TYPE_MONITOR) {
-		text = g_strdup_printf ("%.1f W", obj->energy_rate);
+	if (type == DKP_DEVICE_TYPE_BATTERY ||
+	    type == DKP_DEVICE_TYPE_MONITOR) {
+		text = g_strdup_printf ("%.1f W", energy_rate);
 		gpm_stats_add_info_data (_("Rate"), text);
 		g_free (text);
 	}
-	if (obj->type == DKP_DEVICE_TYPE_UPS ||
-	    obj->type == DKP_DEVICE_TYPE_BATTERY ||
-	    obj->type == DKP_DEVICE_TYPE_MONITOR) {
-		text = g_strdup_printf ("%.1f V", obj->voltage);
+	if (type == DKP_DEVICE_TYPE_UPS ||
+	    type == DKP_DEVICE_TYPE_BATTERY ||
+	    type == DKP_DEVICE_TYPE_MONITOR) {
+		text = g_strdup_printf ("%.1f V", voltage);
 		gpm_stats_add_info_data (_("Voltage"), text);
 		g_free (text);
 	}
-	if (obj->type == DKP_DEVICE_TYPE_BATTERY ||
-	    obj->type == DKP_DEVICE_TYPE_UPS) {
-		if (obj->time_to_full >= 0) {
-			text = gpm_stats_time_to_text (obj->time_to_full);
+	if (type == DKP_DEVICE_TYPE_BATTERY ||
+	    type == DKP_DEVICE_TYPE_UPS) {
+		if (time_to_full >= 0) {
+			text = gpm_stats_time_to_text (time_to_full);
 			gpm_stats_add_info_data (_("Time to full"), text);
 			g_free (text);
 		}
-		if (obj->time_to_empty >= 0) {
-			text = gpm_stats_time_to_text (obj->time_to_empty);
+		if (time_to_empty >= 0) {
+			text = gpm_stats_time_to_text (time_to_empty);
 			gpm_stats_add_info_data (_("Time to empty"), text);
 			g_free (text);
 		}
 	}
-	if (obj->type == DKP_DEVICE_TYPE_BATTERY ||
-	    obj->type == DKP_DEVICE_TYPE_MOUSE ||
-	    obj->type == DKP_DEVICE_TYPE_KEYBOARD ||
-	    obj->type == DKP_DEVICE_TYPE_UPS) {
-		text = g_strdup_printf ("%.1f%%", obj->percentage);
+	if (type == DKP_DEVICE_TYPE_BATTERY ||
+	    type == DKP_DEVICE_TYPE_MOUSE ||
+	    type == DKP_DEVICE_TYPE_KEYBOARD ||
+	    type == DKP_DEVICE_TYPE_UPS) {
+		text = g_strdup_printf ("%.1f%%", percentage);
 		gpm_stats_add_info_data (_("Percentage"), text);
 		g_free (text);
 	}
-	if (obj->type == DKP_DEVICE_TYPE_BATTERY) {
-		text = g_strdup_printf ("%.1f%%", obj->capacity);
+	if (type == DKP_DEVICE_TYPE_BATTERY) {
+		text = g_strdup_printf ("%.1f%%", capacity);
 		gpm_stats_add_info_data (_("Capacity"), text);
 		g_free (text);
 	}
-	if (obj->type == DKP_DEVICE_TYPE_BATTERY)
-		gpm_stats_add_info_data (_("Technology"), gpm_device_technology_to_localised_text (obj->technology));
-	if (obj->type == DKP_DEVICE_TYPE_LINE_POWER)
-		gpm_stats_add_info_data (_("Online"), gpm_stats_bool_to_text (obj->online));
+	if (type == DKP_DEVICE_TYPE_BATTERY)
+		gpm_stats_add_info_data (_("Technology"), gpm_device_technology_to_localised_text (technology));
+	if (type == DKP_DEVICE_TYPE_LINE_POWER)
+		gpm_stats_add_info_data (_("Online"), gpm_stats_bool_to_text (online));
+
+	g_free (vendor);
+	g_free (serial);
+	g_free (model);
 }
 
 /**
  * gpm_stats_set_graph_data:
  **/
 static void
-gpm_stats_set_graph_data (GtkWidget *widget, EggObjList *data, gboolean use_smoothed, gboolean use_points)
+gpm_stats_set_graph_data (GtkWidget *widget, GPtrArray *data, gboolean use_smoothed, gboolean use_points)
 {
-	EggObjList *smoothed;
+	GPtrArray *smoothed;
 
 	gpm_graph_widget_data_clear (GPM_GRAPH_WIDGET (widget));
 
@@ -437,7 +480,8 @@ gpm_stats_set_graph_data (GtkWidget *widget, EggObjList *data, gboolean use_smoo
 		if (use_points)
 			gpm_graph_widget_data_assign (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_PLOT_POINTS, data);
 		gpm_graph_widget_data_assign (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_PLOT_LINE, smoothed);
-		g_object_unref (smoothed);
+		g_ptr_array_foreach (smoothed, (GFunc) gpm_point_obj_free, NULL);
+		g_ptr_array_free (smoothed, TRUE);
 	}
 
 	/* show */
@@ -448,50 +492,67 @@ gpm_stats_set_graph_data (GtkWidget *widget, EggObjList *data, gboolean use_smoo
  * gpm_stats_update_info_page_history:
  **/
 static void
-gpm_stats_update_info_page_history (const DkpDevice *device)
+gpm_stats_update_info_page_history (DkpDevice *device)
 {
-	EggObjList *array;
+	GPtrArray *array;
 	guint i;
 	DkpHistoryObj *hobj;
 	GtkWidget *widget;
 	gboolean checked;
 	gboolean points;
 	GpmPointObj *point;
-	EggObjList *new;
+	GPtrArray *new;
 	gint32 offset = 0;
 	GTimeVal timeval;
 
-	new = egg_obj_list_new ();
-	egg_obj_list_set_copy (new, (EggObjListCopyFunc) dkp_point_obj_copy);
-	egg_obj_list_set_free (new, (EggObjListFreeFunc) dkp_point_obj_free);
+	new = g_ptr_array_new ();
+	if (strcmp (history_type, GPM_HISTORY_CHARGE_VALUE) == 0) {
+		g_object_set (graph_history,
+			      "type-x", GPM_GRAPH_WIDGET_TYPE_TIME,
+			      "type-y", GPM_GRAPH_WIDGET_TYPE_PERCENTAGE,
+			      "autorange-x", FALSE,
+			      "start-x", -history_time,
+			      "stop-x", 0,
+			      "autorange-y", TRUE,
+			      NULL);
+	} else if (strcmp (history_type, GPM_HISTORY_RATE_VALUE) == 0) {
+		g_object_set (graph_history,
+			      "type-x", GPM_GRAPH_WIDGET_TYPE_TIME,
+			      "type-y", GPM_GRAPH_WIDGET_TYPE_POWER,
+			      "autorange-x", FALSE,
+			      "start-x", -history_time,
+			      "stop-x", 0,
+			      "autorange-y", TRUE,
+			      NULL);
+	} else {
+		g_object_set (graph_history,
+			      "type-x", GPM_GRAPH_WIDGET_TYPE_TIME,
+			      "type-y", GPM_GRAPH_WIDGET_TYPE_TIME,
+			      "autorange-x", FALSE,
+			      "start-x", -history_time,
+			      "stop-x", 0,
+			      "autorange-y", TRUE,
+			      NULL);
+	}
 
-	widget = glade_xml_get_widget (glade_xml, "custom_graph_history");
-	gpm_graph_widget_set_type_x (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_TYPE_TIME);
-	if (strcmp (history_type, GPM_HISTORY_CHARGE_VALUE) == 0)
-		gpm_graph_widget_set_type_y (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_TYPE_PERCENTAGE);
-	else if (strcmp (history_type, GPM_HISTORY_RATE_VALUE) == 0)
-		gpm_graph_widget_set_type_y (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_TYPE_POWER);
-	else
-		gpm_graph_widget_set_type_y (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_TYPE_TIME);
-
-	array = dkp_device_get_history (device, history_type, history_time, 150);
+	array = dkp_device_get_history (device, history_type, history_time, 150, NULL);
 	if (array == NULL) {
-		gtk_widget_hide (widget);
+		gtk_widget_hide (graph_history);
 		goto out;
 	}
-	gtk_widget_show (widget);
+	gtk_widget_show (graph_history);
 
 	g_get_current_time (&timeval);
 	offset = timeval.tv_sec;
 
 	for (i=0; i<array->len; i++) {
-		hobj = (DkpHistoryObj *) egg_obj_list_index (array, i);
+		hobj = (DkpHistoryObj *) g_ptr_array_index (array, i);
 
 		/* abandon this point */
 		if (hobj->state == DKP_DEVICE_STATE_UNKNOWN)
 			continue;
 
-		point = dkp_point_obj_new ();
+		point = gpm_point_obj_new ();
 		point->x = (gint32) hobj->time - offset;
 		point->y = hobj->value;
 		if (hobj->state == DKP_DEVICE_STATE_CHARGING)
@@ -504,23 +565,23 @@ gpm_stats_update_info_page_history (const DkpDevice *device)
 			else
 				point->color = egg_color_from_rgb (0, 255, 0);
 		}
-		egg_obj_list_add (new, point);
-		dkp_point_obj_free (point);
+		g_ptr_array_add (new, point);
 	}
 
 	/* render */
 	sigma_smoothing = 2.0;
-	widget = glade_xml_get_widget (glade_xml, "checkbutton_smooth_history");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "checkbutton_smooth_history"));
 	checked = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
-	widget = glade_xml_get_widget (glade_xml, "checkbutton_points_history");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "checkbutton_points_history"));
 	points = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 
 	/* present data to graph */
-	widget = glade_xml_get_widget (glade_xml, "custom_graph_history");
-	gpm_stats_set_graph_data (widget, new, checked, points);
+	gpm_stats_set_graph_data (graph_history, new, checked, points);
 
-	g_object_unref (array);
-	g_object_unref (new);
+	g_ptr_array_foreach (array, (GFunc) dkp_history_obj_free, NULL);
+	g_ptr_array_free (array, TRUE);
+	g_ptr_array_foreach (new, (GFunc) gpm_point_obj_free, NULL);
+	g_ptr_array_free (new, TRUE);
 out:
 	return;
 }
@@ -529,24 +590,20 @@ out:
  * gpm_stats_update_info_page_stats:
  **/
 static void
-gpm_stats_update_info_page_stats (const DkpDevice *device)
+gpm_stats_update_info_page_stats (DkpDevice *device)
 {
-	EggObjList *array;
+	GPtrArray *array;
 	guint i;
 	DkpStatsObj *sobj;
 	GtkWidget *widget;
 	gboolean checked;
 	gboolean points;
 	GpmPointObj *point;
-	EggObjList *new;
+	GPtrArray *new;
 	gboolean use_data = FALSE;
 	const gchar *type = NULL;
 
-	new = egg_obj_list_new ();
-	egg_obj_list_set_copy (new, (EggObjListCopyFunc) dkp_point_obj_copy);
-	egg_obj_list_set_free (new, (EggObjListFreeFunc) dkp_point_obj_free);
-
-	widget = glade_xml_get_widget (glade_xml, "custom_graph_stats");
+	new = g_ptr_array_new ();
 	if (strcmp (stats_type, GPM_STATS_CHARGE_DATA_VALUE) == 0) {
 		type = "charging";
 		use_data = TRUE;
@@ -562,45 +619,55 @@ gpm_stats_update_info_page_stats (const DkpDevice *device)
 	} else {
 		g_assert_not_reached ();
 	}
-	gpm_graph_widget_set_type_x (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_TYPE_PERCENTAGE);
 
-	if (use_data)
-		gpm_graph_widget_set_type_y (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_TYPE_FACTOR);
-	else
-		gpm_graph_widget_set_type_y (GPM_GRAPH_WIDGET (widget), GPM_GRAPH_WIDGET_TYPE_PERCENTAGE);
+	if (use_data) {
+		g_object_set (graph_statistics,
+			      "type-x", GPM_GRAPH_WIDGET_TYPE_PERCENTAGE,
+			      "type-y", GPM_GRAPH_WIDGET_TYPE_FACTOR,
+			      "autorange-x", TRUE,
+			      "autorange-y", TRUE,
+			      NULL);
+	} else {
+		g_object_set (graph_statistics,
+			      "type-x", GPM_GRAPH_WIDGET_TYPE_PERCENTAGE,
+			      "type-y", GPM_GRAPH_WIDGET_TYPE_PERCENTAGE,
+			      "autorange-x", TRUE,
+			      "autorange-y", TRUE,
+			      NULL);
+	}
 
-	array = dkp_device_get_statistics (device, type);
+	array = dkp_device_get_statistics (device, type, NULL);
 	if (array == NULL) {
-		gtk_widget_hide (widget);
+		gtk_widget_hide (graph_statistics);
 		goto out;
 	}
 
 	for (i=0; i<array->len; i++) {
-		sobj = (DkpStatsObj *) egg_obj_list_index (array, i);
-		point = dkp_point_obj_new ();
+		sobj = (DkpStatsObj *) g_ptr_array_index (array, i);
+		point = gpm_point_obj_new ();
 		point->x = i;
 		if (use_data)
 			point->y = sobj->value;
 		else
 			point->y = sobj->accuracy;
 		point->color = egg_color_from_rgb (255, 0, 0);
-		egg_obj_list_add (new, point);
-		dkp_point_obj_free (point);
+		g_ptr_array_add (new, point);
 	}
 
 	/* render */
 	sigma_smoothing = 1.1;
-	widget = glade_xml_get_widget (glade_xml, "checkbutton_smooth_stats");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "checkbutton_smooth_stats"));
 	checked = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
-	widget = glade_xml_get_widget (glade_xml, "checkbutton_points_stats");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "checkbutton_points_stats"));
 	points = gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (widget));
 
 	/* present data to graph */
-	widget = glade_xml_get_widget (glade_xml, "custom_graph_stats");
-	gpm_stats_set_graph_data (widget, new, checked, points);
+	gpm_stats_set_graph_data (graph_statistics, new, checked, points);
 
-	g_object_unref (array);
-	g_object_unref (new);
+	g_ptr_array_foreach (array, (GFunc) dkp_stats_obj_free, NULL);
+	g_ptr_array_free (array, TRUE);
+	g_ptr_array_foreach (new, (GFunc) gpm_point_obj_free, NULL);
+	g_ptr_array_free (new, TRUE);
 out:
 	return;
 }
@@ -609,7 +676,7 @@ out:
  * gpm_stats_update_info_data_page:
  **/
 static void
-gpm_stats_update_info_data_page (const DkpDevice *device, gint page)
+gpm_stats_update_info_data_page (DkpDevice *device, gint page)
 {
 	if (page == 0)
 		gpm_stats_update_info_page_details (device);
@@ -623,39 +690,46 @@ gpm_stats_update_info_data_page (const DkpDevice *device, gint page)
  * gpm_stats_update_info_data:
  **/
 static void
-gpm_stats_update_info_data (const DkpDevice *device)
+gpm_stats_update_info_data (DkpDevice *device)
 {
 	gint page;
-	GtkWidget *widget;
+	GtkNotebook *notebook;
 	GtkWidget *page_widget;
-	const DkpObject	*obj;
+	gboolean has_history;
+	gboolean has_statistics;
 
-	widget = glade_xml_get_widget (glade_xml, "notebook1");
-	obj = dkp_device_get_object (device);
+	/* get properties */
+	g_object_get (device,
+		      "has-history", &has_history,
+		      "has-statistics", &has_statistics,
+		      NULL);
+
+
+	notebook = GTK_NOTEBOOK (gtk_builder_get_object (builder, "notebook1"));
 
 	/* show info page */
-	page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK(widget), 0);
+	page_widget = gtk_notebook_get_nth_page (notebook, 0);
 	gtk_widget_show (page_widget);
 
 	/* hide history if no support */
-	page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK(widget), 1);
-	if (obj->has_history)
+	page_widget = gtk_notebook_get_nth_page (notebook, 1);
+	if (has_history)
 		gtk_widget_show (page_widget);
 	else
 		gtk_widget_hide (page_widget);
 
 	/* hide statistics if no support */
-	page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK(widget), 2);
-	if (obj->has_statistics)
+	page_widget = gtk_notebook_get_nth_page (notebook, 2);
+	if (has_statistics)
 		gtk_widget_show (page_widget);
 	else
 		gtk_widget_hide (page_widget);
 
 	/* hide wakeups page */
-	page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK(widget), 3);
+	page_widget = gtk_notebook_get_nth_page (notebook, 3);
 	gtk_widget_hide (page_widget);
 
-	page = gtk_notebook_get_current_page (GTK_NOTEBOOK (widget));
+	page = gtk_notebook_get_current_page (notebook);
 	gpm_stats_update_info_data_page (device, page);
 
 	return;
@@ -869,7 +943,7 @@ gpm_stats_update_wakeups_data (void)
 	GError *error = NULL;
 	GPtrArray *array;
 
-	widget = glade_xml_get_widget (glade_xml, "notebook1");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "notebook1"));
 
 	/* hide other pages */
 	page_widget = gtk_notebook_get_nth_page (GTK_NOTEBOOK(widget), 0);
@@ -885,7 +959,7 @@ gpm_stats_update_wakeups_data (void)
 
 	/* show total */
 	total = dkp_wakeups_get_total (wakeups, &error);
-	widget = glade_xml_get_widget (glade_xml, "label_total_wakeups");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label_total_wakeups"));
 	if (error == NULL) {
 		text = g_strdup_printf ("%i", total);
 		gtk_label_set_label (GTK_LABEL(widget), text);
@@ -939,7 +1013,7 @@ gpm_stats_notebook_changed_cb (GtkNotebook *notebook, GtkNotebookPage *page, gin
 	GtkWidget *widget;
 
 	/* set the window title depending on the mode */
-	widget = glade_xml_get_widget (glade_xml, "dialog_stats");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_stats"));
 	gpm_stats_set_title (GTK_WINDOW (widget), page_num);
 
 	/* save page in gconf */
@@ -948,11 +1022,11 @@ gpm_stats_notebook_changed_cb (GtkNotebook *notebook, GtkNotebookPage *page, gin
 	if (current_device == NULL)
 		return;
 
-	if (egg_strequal (current_device, "wakeups"))
+	if (g_strcmp0 (current_device, "wakeups") == 0)
 		return;
 
 	device = dkp_device_new ();
-	dkp_device_set_object_path (device, current_device);
+	dkp_device_set_object_path (device, current_device, NULL);
 	gpm_stats_update_info_data_page (device, page_num);
 	gpm_stats_update_info_data (device);
 	g_object_unref (device);
@@ -966,7 +1040,7 @@ gpm_stats_button_update_ui (void)
 {
 	DkpDevice *device;
 	device = dkp_device_new ();
-	dkp_device_set_object_path (device, current_device);
+	dkp_device_set_object_path (device, current_device, NULL);
 	gpm_stats_update_info_data (device);
 	g_object_unref (device);
 }
@@ -993,11 +1067,11 @@ gpm_stats_devices_treeview_clicked_cb (GtkTreeSelection *selection, gboolean dat
 		egg_debug ("selected row is: %s", current_device);
 
 		/* is special device */
-		if (egg_strequal (current_device, "wakeups")) {
+		if (g_strcmp0 (current_device, "wakeups") == 0) {
 			gpm_stats_update_wakeups_data ();
 		} else {
 			device = dkp_device_new ();
-			dkp_device_set_object_path (device, current_device);
+			dkp_device_set_object_path (device, current_device, NULL);
 			gpm_stats_update_info_data (device);
 			g_object_unref (device);
 		}
@@ -1007,22 +1081,6 @@ gpm_stats_devices_treeview_clicked_cb (GtkTreeSelection *selection, gboolean dat
 	}
 }
 
-/**
- * gpm_stats_create_custom_widget:
- **/
-static GtkWidget *
-gpm_stats_create_custom_widget (GladeXML *xml, gchar *func_name, gchar *name,
-				gchar *string1, gchar *string2, gint int1, gint int2, gpointer user_data)
-{
-	GtkWidget *widget = NULL;
-	if (strcmp ("gpm_graph_widget_new", func_name) == 0) {
-		widget = gpm_graph_widget_new ();
-		return widget;
-	}
-	egg_warning ("name unknown=%s", name);
-	return NULL;
-}
-
 /**
  * gpm_stats_window_activated_cb
  **/
@@ -1030,7 +1088,7 @@ static void
 gpm_stats_window_activated_cb (EggUnique *egg_unique, gpointer data)
 {
 	GtkWidget *widget;
-	widget = glade_xml_get_widget (glade_xml, "dialog_stats");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_stats"));
 	gtk_window_present (GTK_WINDOW (widget));
 }
 
@@ -1038,18 +1096,22 @@ gpm_stats_window_activated_cb (EggUnique *egg_unique, gpointer data)
  * gpm_stats_add_device:
  **/
 static void
-gpm_stats_add_device (const DkpDevice *device)
+gpm_stats_add_device (DkpDevice *device)
 {
 	const gchar *id;
 	GtkTreeIter iter;
-	const DkpObject *obj;
 	const gchar *text;
 	const gchar *icon;
+	DkpDeviceType type;
+
+	/* get device properties */
+	g_object_get (device,
+		      "type", &type,
+		      NULL);
 
-	obj = dkp_device_get_object (device);
 	id = dkp_device_get_object_path (device);
-	text = gpm_device_type_to_localised_text (obj->type, 1);
-	icon = gpm_devicekit_get_object_icon (obj);
+	text = gpm_device_type_to_localised_text (type, 1);
+	icon = gpm_devicekit_get_object_icon (device);
 
 	gtk_list_store_append (list_store_devices, &iter);
 	gtk_list_store_set (list_store_devices, &iter,
@@ -1064,7 +1126,7 @@ gpm_stats_add_device (const DkpDevice *device)
 static void
 gpm_stats_data_changed_cb (DkpClient *client, gpointer user_data)
 {
-	if (egg_strequal (current_device, "wakeups"))
+	if (g_strcmp0 (current_device, "wakeups") == 0)
 		gpm_stats_update_wakeups_data ();
 }
 
@@ -1072,7 +1134,7 @@ gpm_stats_data_changed_cb (DkpClient *client, gpointer user_data)
  * gpm_stats_device_added_cb:
  **/
 static void
-gpm_stats_device_added_cb (DkpClient *client, const DkpDevice *device, gpointer user_data)
+gpm_stats_device_added_cb (DkpClient *client, DkpDevice *device, gpointer user_data)
 {
 	const gchar *object_path;
 	object_path = dkp_device_get_object_path (device);
@@ -1084,7 +1146,7 @@ gpm_stats_device_added_cb (DkpClient *client, const DkpDevice *device, gpointer
  * gpm_stats_device_changed_cb:
  **/
 static void
-gpm_stats_device_changed_cb (DkpClient *client, const DkpDevice *device, gpointer user_data)
+gpm_stats_device_changed_cb (DkpClient *client, DkpDevice *device, gpointer user_data)
 {
 	const gchar *object_path;
 	object_path = dkp_device_get_object_path (device);
@@ -1099,7 +1161,7 @@ gpm_stats_device_changed_cb (DkpClient *client, const DkpDevice *device, gpointe
  * gpm_stats_device_removed_cb:
  **/
 static void
-gpm_stats_device_removed_cb (DkpClient *client, const DkpDevice *device, gpointer user_data)
+gpm_stats_device_removed_cb (DkpClient *client, DkpDevice *device, gpointer user_data)
 {
 	const gchar *object_path;
 	GtkTreeIter iter;
@@ -1164,9 +1226,9 @@ gpm_stats_history_type_combo_changed_cb (GtkWidget *widget, gpointer data)
 	}
 
 	/* set axis */
-	widget = glade_xml_get_widget (glade_xml, "label_axis_history_x");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label_axis_history_x"));
 	gtk_label_set_label (GTK_LABEL(widget), axis_x);
-	widget = glade_xml_get_widget (glade_xml, "label_axis_history_y");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label_axis_history_y"));
 	gtk_label_set_label (GTK_LABEL(widget), axis_y);
 
 	gpm_stats_button_update_ui ();
@@ -1215,9 +1277,9 @@ gpm_stats_type_combo_changed_cb (GtkWidget *widget, gpointer data)
 	}
 
 	/* set axis */
-	widget = glade_xml_get_widget (glade_xml, "label_axis_stats_x");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label_axis_stats_x"));
 	gtk_label_set_label (GTK_LABEL(widget), axis_x);
-	widget = glade_xml_get_widget (glade_xml, "label_axis_stats_y");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "label_axis_stats_y"));
 	gtk_label_set_label (GTK_LABEL(widget), axis_y);
 
 	gpm_stats_button_update_ui ();
@@ -1305,6 +1367,26 @@ gpm_stats_points_checkbox_stats_cb (GtkWidget *widget, gpointer data)
 	gpm_stats_button_update_ui ();
 }
 
+/**
+ * gpm_stats_set_combo_simple_text:
+ **/
+static void
+gpm_stats_set_combo_simple_text (GtkWidget *combo_box)
+{
+	GtkCellRenderer *cell;
+	GtkListStore *store;
+
+	store = gtk_list_store_new (1, G_TYPE_STRING);
+	gtk_combo_box_set_model (GTK_COMBO_BOX (combo_box), GTK_TREE_MODEL (store));
+	g_object_unref (store);
+
+	cell = gtk_cell_renderer_text_new ();
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (combo_box), cell, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (combo_box), cell,
+					"text", 0,
+					NULL);
+}
+
 /**
  * main:
  **/
@@ -1313,6 +1395,7 @@ main (int argc, char *argv[])
 {
 	gboolean verbose = FALSE;
 	GOptionContext *context;
+	GtkBox *box;
 	GtkWidget *widget;
 	GtkTreeSelection *selection;
 	EggUnique *egg_unique;
@@ -1325,6 +1408,8 @@ main (int argc, char *argv[])
 	const gchar *object_path;
 	gboolean checked;
 	gchar *last_device;
+	guint retval;
+	GError *error = NULL;
 
 	const GOptionEntry options[] = {
 		{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose,
@@ -1373,50 +1458,68 @@ main (int argc, char *argv[])
 	gtk_icon_theme_append_search_path (gtk_icon_theme_get_default (),
                                            GPM_DATA G_DIR_SEPARATOR_S "icons");
 
-	/* use custom widgets */
-	glade_set_custom_handler (gpm_stats_create_custom_widget, NULL);
+	/* get UI */
+	builder = gtk_builder_new ();
+	retval = gtk_builder_add_from_file (builder, GPM_DATA "/gpm-statistics.ui", &error);
+	if (error != NULL) {
+		egg_warning ("failed to load ui: %s", error->message);
+		g_error_free (error);
+	}
 
-	glade_xml = glade_xml_new (GPM_DATA "/gpm-statistics.glade", NULL, NULL);
-	widget = glade_xml_get_widget (glade_xml, "dialog_stats");
+	/* add history graph */
+	box = GTK_BOX (gtk_builder_get_object (builder, "hbox_history"));
+	graph_history = gpm_graph_widget_new ();
+	gtk_box_pack_start (box, graph_history, TRUE, TRUE, 0);
+	gtk_widget_set_size_request (graph_history, 600, 300);
+	gtk_widget_show (graph_history);
+
+	/* add statistics graph */
+	box = GTK_BOX (gtk_builder_get_object (builder, "hbox_statistics"));
+	graph_statistics = gpm_graph_widget_new ();
+	gtk_box_pack_start (box, graph_statistics, TRUE, TRUE, 0);
+	gtk_widget_set_size_request (graph_statistics, 600, 300);
+	gtk_widget_show (graph_statistics);
+
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_stats"));
 	gtk_window_set_default_size (GTK_WINDOW(widget), 800, 500);
 	gtk_window_set_default_icon_name (GPM_STOCK_APP_ICON);
 
 	/* Get the main window quit */
 	g_signal_connect_swapped (widget, "delete_event", G_CALLBACK (gtk_main_quit), NULL);
 
-	widget = glade_xml_get_widget (glade_xml, "button_close");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_close"));
 	g_signal_connect_swapped (widget, "clicked", G_CALLBACK (gtk_main_quit), NULL);
 	gtk_widget_grab_default (widget);
 
-	widget = glade_xml_get_widget (glade_xml, "button_help");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "button_help"));
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_stats_button_help_cb), NULL);
 
-	widget = glade_xml_get_widget (glade_xml, "checkbutton_smooth_history");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "checkbutton_smooth_history"));
 	checked = gconf_client_get_bool (gconf_client, GPM_CONF_INFO_HISTORY_GRAPH_SMOOTH, NULL);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), checked);
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_stats_smooth_checkbox_history_cb), NULL);
 
-	widget = glade_xml_get_widget (glade_xml, "checkbutton_smooth_stats");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "checkbutton_smooth_stats"));
 	checked = gconf_client_get_bool (gconf_client, GPM_CONF_INFO_STATS_GRAPH_SMOOTH, NULL);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), checked);
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_stats_smooth_checkbox_stats_cb), NULL);
 
-	widget = glade_xml_get_widget (glade_xml, "checkbutton_points_history");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "checkbutton_points_history"));
 	checked = gconf_client_get_bool (gconf_client, GPM_CONF_INFO_HISTORY_GRAPH_POINTS, NULL);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), checked);
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_stats_points_checkbox_history_cb), NULL);
 
-	widget = glade_xml_get_widget (glade_xml, "checkbutton_points_stats");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "checkbutton_points_stats"));
 	checked = gconf_client_get_bool (gconf_client, GPM_CONF_INFO_STATS_GRAPH_POINTS, NULL);
 	gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), checked);
 	g_signal_connect (widget, "clicked",
 			  G_CALLBACK (gpm_stats_points_checkbox_stats_cb), NULL);
 
-	widget = glade_xml_get_widget (glade_xml, "notebook1");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "notebook1"));
 	page = gconf_client_get_int (gconf_client, GPM_CONF_INFO_PAGE_NUMBER, NULL);
 	gtk_notebook_set_current_page (GTK_NOTEBOOK (widget), page);
 	g_signal_connect (widget, "switch-page",
@@ -1430,7 +1533,7 @@ main (int argc, char *argv[])
 						 G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
 
 	/* create transaction_id tree view */
-	widget = glade_xml_get_widget (glade_xml, "treeview_info");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_info"));
 	gtk_tree_view_set_model (GTK_TREE_VIEW (widget),
 				 GTK_TREE_MODEL (list_store_info));
 
@@ -1439,7 +1542,7 @@ main (int argc, char *argv[])
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (widget)); /* show */
 
 	/* create transaction_id tree view */
-	widget = glade_xml_get_widget (glade_xml, "treeview_devices");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_devices"));
 	gtk_tree_view_set_model (GTK_TREE_VIEW (widget),
 				 GTK_TREE_MODEL (list_store_devices));
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
@@ -1451,7 +1554,7 @@ main (int argc, char *argv[])
 	gtk_tree_view_columns_autosize (GTK_TREE_VIEW (widget)); /* show */
 
 	/* create wakeups tree view */
-	widget = glade_xml_get_widget (glade_xml, "treeview_wakeups");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_wakeups"));
 	gtk_tree_view_set_model (GTK_TREE_VIEW (widget),
 				 GTK_TREE_MODEL (list_store_wakeups));
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
@@ -1471,7 +1574,8 @@ main (int argc, char *argv[])
 	if (stats_type == NULL)
 		stats_type = GPM_STATS_CHARGE_DATA_VALUE;
 
-	widget = glade_xml_get_widget (glade_xml, "combobox_history_type");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_history_type"));
+	gpm_stats_set_combo_simple_text (widget);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_HISTORY_RATE_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_HISTORY_CHARGE_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_HISTORY_TIME_FULL_TEXT);
@@ -1483,7 +1587,8 @@ main (int argc, char *argv[])
 	g_signal_connect (G_OBJECT (widget), "changed",
 			  G_CALLBACK (gpm_stats_history_type_combo_changed_cb), NULL);
 
-	widget = glade_xml_get_widget (glade_xml, "combobox_stats_type");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_stats_type"));
+	gpm_stats_set_combo_simple_text (widget);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_STATS_CHARGE_DATA_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_STATS_CHARGE_ACCURACY_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_STATS_DISCHARGE_DATA_TEXT);
@@ -1501,7 +1606,8 @@ main (int argc, char *argv[])
 	g_signal_connect (G_OBJECT (widget), "changed",
 			  G_CALLBACK (gpm_stats_type_combo_changed_cb), NULL);
 
-	widget = glade_xml_get_widget (glade_xml, "combobox_history_time");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_history_time"));
+	gpm_stats_set_combo_simple_text (widget);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_HISTORY_MINUTE_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_HISTORY_HOUR_TEXT);
 	gtk_combo_box_append_text (GTK_COMBO_BOX (widget), GPM_HISTORY_DAY_TEXT);
@@ -1516,13 +1622,6 @@ main (int argc, char *argv[])
 	g_signal_connect (G_OBJECT (widget), "changed",
 			  G_CALLBACK (gpm_stats_range_combo_changed), NULL);
 
-	widget = glade_xml_get_widget (glade_xml, "custom_graph_history");
-	gtk_widget_set_size_request (widget, 700, 400);
-	gtk_widget_show (widget);
-	widget = glade_xml_get_widget (glade_xml, "custom_graph_stats");
-	gtk_widget_set_size_request (widget, 700, 400);
-	gtk_widget_show (widget);
-
 	client = dkp_client_new ();
 	g_signal_connect (client, "device-added", G_CALLBACK (gpm_stats_device_added_cb), NULL);
 	g_signal_connect (client, "device-removed", G_CALLBACK (gpm_stats_device_removed_cb), NULL);
@@ -1532,7 +1631,7 @@ main (int argc, char *argv[])
 	g_signal_connect (wakeups, "data-changed", G_CALLBACK (gpm_stats_data_changed_cb), NULL);
 
 	/* coldplug */
-	devices = dkp_client_enumerate_devices (client);
+	devices = dkp_client_enumerate_devices (client, NULL);
 	if (devices == NULL)
 		goto out;
 	for (i=0; i < devices->len; i++) {
@@ -1546,9 +1645,14 @@ main (int argc, char *argv[])
 
 	last_device = gconf_client_get_string (gconf_client, GPM_CONF_INFO_LAST_DEVICE, NULL);
 
+#if 0
+	/* FIXME: this is only available in DeviceKit 008 */
+	ret = dkp_wakeups_has_capability (wakeups);
+#else
 	/* can we get wakeup data? */
-	i = dkp_wakeups_get_total (wakeups, NULL);
-	if (i > 0) {
+	ret = TRUE;
+#endif
+	if (ret) {
 		GtkTreeIter iter;
 		gtk_list_store_append (list_store_devices, &iter);
 		gtk_list_store_set (list_store_devices, &iter,
@@ -1568,7 +1672,7 @@ main (int argc, char *argv[])
 			gchar *path_str;
 			path_str = g_strdup_printf ("%i", i);
 			path = gtk_tree_path_new_from_string (path_str);
-			widget = glade_xml_get_widget (glade_xml, "treeview_devices");
+			widget = GTK_WIDGET (gtk_builder_get_object (builder, "treeview_devices"));
 			gtk_tree_view_set_cursor_on_cell (GTK_TREE_VIEW (widget), path, NULL, NULL, FALSE);
 			g_free (path_str);
 			gtk_tree_path_free (path);
@@ -1579,12 +1683,12 @@ main (int argc, char *argv[])
 	g_ptr_array_free (devices, TRUE);
 
 	/* set axis */
-	widget = glade_xml_get_widget (glade_xml, "combobox_history_type");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_history_type"));
 	gpm_stats_history_type_combo_changed_cb (widget, NULL);
-	widget = glade_xml_get_widget (glade_xml, "combobox_stats_type");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "combobox_stats_type"));
 	gpm_stats_type_combo_changed_cb (widget, NULL);
 
-	widget = glade_xml_get_widget (glade_xml, "dialog_stats");
+	widget = GTK_WIDGET (gtk_builder_get_object (builder, "dialog_stats"));
 	gtk_widget_show (widget);
 
 	gtk_main ();
@@ -1593,7 +1697,7 @@ out:
 	g_object_unref (gconf_client);
 	g_object_unref (client);
 	g_object_unref (wakeups);
-	g_object_unref (glade_xml);
+	g_object_unref (builder);
 	g_object_unref (list_store_info);
 unique_out:
 	g_object_unref (egg_unique);
